package javax.swing.text;

import java.awt.Container;
import javax.swing.event.DocumentEvent;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.Shape;
import java.awt.Graphics;
import javax.swing.SwingConstants;

public abstract class View implements SwingConstants
{
    public static final int BadBreakWeight = 0;
    public static final int GoodBreakWeight = 1000;
    public static final int ExcellentBreakWeight = 2000;
    public static final int ForcedBreakWeight = 3000;
    public static final int X_AXIS = 0;
    public static final int Y_AXIS = 1;
    static final Position.Bias[] sharedBiasReturn;
    private View parent;
    private Element elem;
    int firstUpdateIndex;
    int lastUpdateIndex;
    
    public View(Element elem) {
        this.elem = elem;
    }
    
    public View getParent() {
        return this.parent;
    }
    
    public boolean isVisible() {
        return true;
    }
    
    public abstract float getPreferredSpan(int p0);
    
    public float getMinimumSpan(int n) {
        if (this.getResizeWeight(n) == 0) {
            return this.getPreferredSpan(n);
        }
        return 0.0f;
    }
    
    public float getMaximumSpan(int n) {
        if (this.getResizeWeight(n) == 0) {
            return this.getPreferredSpan(n);
        }
        return 2.1474836E9f;
    }
    
    public void preferenceChanged(View view, boolean b, boolean b2) {
        View parent = this.getParent();
        if (parent != null) {
            parent.preferenceChanged(this, b, b2);
        }
    }
    
    public float getAlignment(int n) {
        return 0.5f;
    }
    
    public abstract void paint(Graphics p0, Shape p1);
    
    public void setParent(View parent) {
        if (parent == null) {
            for (int i = 0; i < this.getViewCount(); ++i) {
                if (this.getView(i).getParent() == this) {
                    this.getView(i).setParent(null);
                }
            }
        }
        this.parent = parent;
    }
    
    public int getViewCount() {
        return 0;
    }
    
    public View getView(int n) {
        return null;
    }
    
    public void removeAll() {
        this.replace(0, this.getViewCount(), (View[])null);
    }
    
    public void remove(int n) {
        this.replace(n, 1, (View[])null);
    }
    
    public void insert(int n, View view) {
        this.replace(n, 0, new View[] { view });
    }
    
    public void append(View view) {
        this.replace(this.getViewCount(), 0, new View[] { view });
    }
    
    public void replace(int n, int n2, View[] array) {
    }
    
    public int getViewIndex(int n, Position.Bias bias) {
        return -1;
    }
    
    public Shape getChildAllocation(int n, Shape shape) {
        return null;
    }
    
    public int getNextVisualPositionFrom(int n, Position.Bias bias, Shape shape, int i, Position.Bias[] array) throws BadLocationException {
        if (n < -1) {
            throw new BadLocationException("Invalid position", n);
        }
        array[0] = Position.Bias.Forward;
        switch (i) {
            case 1:
            case 5: {
                if (n == -1) {
                    n = ((i == 1) ? Math.max(0, this.getEndOffset() - 1) : this.getStartOffset());
                    break;
                }
                JTextComponent textComponent = (JTextComponent)this.getContainer();
                Caret caret = (textComponent != null) ? textComponent.getCaret() : null;
                Point magicCaretPosition;
                if (caret != null) {
                    magicCaretPosition = caret.getMagicCaretPosition();
                }
                else {
                    magicCaretPosition = null;
                }
                int x;
                if (magicCaretPosition == null) {
                    Rectangle modelToView = textComponent.modelToView(n);
                    x = ((modelToView == null) ? 0 : modelToView.x);
                }
                else {
                    x = magicCaretPosition.x;
                }
                if (i == 1) {
                    n = Utilities.getPositionAbove(textComponent, n, x);
                }
                else {
                    n = Utilities.getPositionBelow(textComponent, n, x);
                }
                break;
            }
            case 7: {
                if (n == -1) {
                    n = Math.max(0, this.getEndOffset() - 1);
                    break;
                }
                n = Math.max(0, n - 1);
                break;
            }
            case 3: {
                if (n == -1) {
                    n = this.getStartOffset();
                    break;
                }
                n = Math.min(n + 1, this.getDocument().getLength());
                break;
            }
            default: {
                throw new IllegalArgumentException("Bad direction: " + i);
            }
        }
        return n;
    }
    
    public abstract Shape modelToView(int p0, Shape p1, Position.Bias p2) throws BadLocationException;
    
    public Shape modelToView(int n, Position.Bias bias, int n2, Position.Bias bias2, Shape shape) throws BadLocationException {
        Shape modelToView = this.modelToView(n, shape, bias);
        Object o;
        if (n2 == this.getEndOffset()) {
            try {
                o = this.modelToView(n2, shape, bias2);
            }
            catch (BadLocationException ex) {
                o = null;
            }
            if (o == null) {
                Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
                o = new Rectangle(rectangle.x + rectangle.width - 1, rectangle.y, 1, rectangle.height);
            }
        }
        else {
            o = this.modelToView(n2, shape, bias2);
        }
        Rectangle bounds = modelToView.getBounds();
        Rectangle r = (o instanceof Rectangle) ? ((Rectangle)o) : o.getBounds();
        if (bounds.y != r.y) {
            Rectangle rectangle2 = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
            bounds.x = rectangle2.x;
            bounds.width = rectangle2.width;
        }
        bounds.add(r);
        return bounds;
    }
    
    public abstract int viewToModel(float p0, float p1, Shape p2, Position.Bias[] p3);
    
    public void insertUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        if (this.getViewCount() > 0) {
            DocumentEvent.ElementChange change = documentEvent.getChange(this.getElement());
            if (change != null && !this.updateChildren(change, documentEvent, viewFactory)) {
                change = null;
            }
            this.forwardUpdate(change, documentEvent, shape, viewFactory);
            this.updateLayout(change, documentEvent, shape);
        }
    }
    
    public void removeUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        if (this.getViewCount() > 0) {
            DocumentEvent.ElementChange change = documentEvent.getChange(this.getElement());
            if (change != null && !this.updateChildren(change, documentEvent, viewFactory)) {
                change = null;
            }
            this.forwardUpdate(change, documentEvent, shape, viewFactory);
            this.updateLayout(change, documentEvent, shape);
        }
    }
    
    public void changedUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        if (this.getViewCount() > 0) {
            DocumentEvent.ElementChange change = documentEvent.getChange(this.getElement());
            if (change != null && !this.updateChildren(change, documentEvent, viewFactory)) {
                change = null;
            }
            this.forwardUpdate(change, documentEvent, shape, viewFactory);
            this.updateLayout(change, documentEvent, shape);
        }
    }
    
    public Document getDocument() {
        return this.elem.getDocument();
    }
    
    public int getStartOffset() {
        return this.elem.getStartOffset();
    }
    
    public int getEndOffset() {
        return this.elem.getEndOffset();
    }
    
    public Element getElement() {
        return this.elem;
    }
    
    public Graphics getGraphics() {
        return this.getContainer().getGraphics();
    }
    
    public AttributeSet getAttributes() {
        return this.elem.getAttributes();
    }
    
    public View breakView(int n, int n2, float n3, float n4) {
        return this;
    }
    
    public View createFragment(int n, int n2) {
        return this;
    }
    
    public int getBreakWeight(int n, float n2, float n3) {
        if (n3 > this.getPreferredSpan(n)) {
            return 1000;
        }
        return 0;
    }
    
    public int getResizeWeight(int n) {
        return 0;
    }
    
    public void setSize(float n, float n2) {
    }
    
    public Container getContainer() {
        View parent = this.getParent();
        return (parent != null) ? parent.getContainer() : null;
    }
    
    public ViewFactory getViewFactory() {
        View parent = this.getParent();
        return (parent != null) ? parent.getViewFactory() : null;
    }
    
    public String getToolTipText(float n, float n2, Shape childAllocation) {
        int viewIndex = this.getViewIndex(n, n2, childAllocation);
        if (viewIndex >= 0) {
            childAllocation = this.getChildAllocation(viewIndex, childAllocation);
            if (((childAllocation instanceof Rectangle) ? ((Rectangle)childAllocation) : childAllocation.getBounds()).contains((double)n, (double)n2)) {
                return this.getView(viewIndex).getToolTipText(n, n2, childAllocation);
            }
        }
        return null;
    }
    
    public int getViewIndex(float n, float n2, Shape shape) {
        int i = this.getViewCount() - 1;
        while (i >= 0) {
            Shape childAllocation = this.getChildAllocation(i, shape);
            if (childAllocation != null && ((childAllocation instanceof Rectangle) ? ((Rectangle)childAllocation) : childAllocation.getBounds()).contains((double)n, (double)n2)) {
                return i;
            }
            --i;
        }
        return -1;
    }
    
    protected boolean updateChildren(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, ViewFactory viewFactory) {
        Element[] childrenRemoved = elementChange.getChildrenRemoved();
        Element[] childrenAdded = elementChange.getChildrenAdded();
        View[] array = null;
        if (childrenAdded != null) {
            array = new View[childrenAdded.length];
            for (int i = 0; i < childrenAdded.length; ++i) {
                array[i] = viewFactory.create(childrenAdded[i]);
            }
        }
        int length = 0;
        int index = elementChange.getIndex();
        if (childrenRemoved != null) {
            length = childrenRemoved.length;
        }
        this.replace(index, length, array);
        return true;
    }
    
    protected void forwardUpdate(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.calculateUpdateIndexes(documentEvent);
        int index = this.lastUpdateIndex + 1;
        Element[] array = (elementChange != null) ? elementChange.getChildrenAdded() : null;
        if (array != null && array.length > 0) {
            index = elementChange.getIndex();
            index = index + array.length - 1;
        }
        for (int i = this.firstUpdateIndex; i <= this.lastUpdateIndex; ++i) {
            if (i < index || i > index) {
                View view = this.getView(i);
                if (view != null) {
                    this.forwardUpdateToView(view, documentEvent, this.getChildAllocation(i, shape), viewFactory);
                }
            }
        }
    }
    
    void calculateUpdateIndexes(DocumentEvent documentEvent) {
        int offset = documentEvent.getOffset();
        this.firstUpdateIndex = this.getViewIndex(offset, Position.Bias.Forward);
        if (this.firstUpdateIndex == -1 && documentEvent.getType() == DocumentEvent.EventType.REMOVE && offset >= this.getEndOffset()) {
            this.firstUpdateIndex = this.getViewCount() - 1;
        }
        this.lastUpdateIndex = this.firstUpdateIndex;
        View view = (this.firstUpdateIndex >= 0) ? this.getView(this.firstUpdateIndex) : null;
        if (view != null && view.getStartOffset() == offset && offset > 0) {
            this.firstUpdateIndex = Math.max(this.firstUpdateIndex - 1, 0);
        }
        if (documentEvent.getType() != DocumentEvent.EventType.REMOVE) {
            this.lastUpdateIndex = this.getViewIndex(offset + documentEvent.getLength(), Position.Bias.Forward);
            if (this.lastUpdateIndex < 0) {
                this.lastUpdateIndex = this.getViewCount() - 1;
            }
        }
        this.firstUpdateIndex = Math.max(this.firstUpdateIndex, 0);
    }
    
    void updateAfterChange() {
    }
    
    protected void forwardUpdateToView(View view, DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        DocumentEvent.EventType type = documentEvent.getType();
        if (type == DocumentEvent.EventType.INSERT) {
            view.insertUpdate(documentEvent, shape, viewFactory);
        }
        else if (type == DocumentEvent.EventType.REMOVE) {
            view.removeUpdate(documentEvent, shape, viewFactory);
        }
        else {
            view.changedUpdate(documentEvent, shape, viewFactory);
        }
    }
    
    protected void updateLayout(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, Shape shape) {
        if (elementChange != null && shape != null) {
            this.preferenceChanged(null, true, true);
            Container container = this.getContainer();
            if (container != null) {
                container.repaint();
            }
        }
    }
    
    @Deprecated
    public Shape modelToView(int n, Shape shape) throws BadLocationException {
        return this.modelToView(n, shape, Position.Bias.Forward);
    }
    
    @Deprecated
    public int viewToModel(float n, float n2, Shape shape) {
        View.sharedBiasReturn[0] = Position.Bias.Forward;
        return this.viewToModel(n, n2, shape, View.sharedBiasReturn);
    }
    
    static {
        sharedBiasReturn = new Position.Bias[1];
    }
}
